#!/bin/bash

source `which setPortNumbers.sh`

AGGREGATOR_NODE="localhost"

# this function expects a number of arguments:
#  1) the DAQ command to be sent
#  2) the run number (dummy for non-start commands)
#  3) the pulser trigger rate
#  4) the laser trigger rate
#  5) whether to enable the random trigger (0 or 1)
#  6) the acqusition gate width (usec)
#  7) the majority logic low threshold
#  8) the compression level for ADC data [0..2]
#  9) whether to run online monitoring
# 10) the data directory
# 11) the logfile name
# 12) whether to write data to disk [0,1]
# 13) the desired number of events in the run (stop command)
# 14) the desired duration of the run (minutes, stop command)
# 15) the desired size of each data file
# 16) the pre-trigger window size (usec)
# 17) the desired number of events in each file
# 18) the desired time duration of each file (minutes)
function launch() {
  ebComp=$8
  agComp=$8
  if [[ "$8" == "2" ]]; then
    ebComp=1
    agComp=3
  elif [[ "$8" == "1" ]]; then
    ebComp=1
    agComp=0
  fi

  ds50MasterControl.rb -s -c $1 \
    --v1720 localhost,$frPort0,1,1,$6,${16} \
    --eb localhost,$ebPort0,$ebComp \
    --eb localhost,$ebPort1,$ebComp \
    --ag ${AGGREGATOR_NODE},$agPort0,1,$agComp \
    --data-dir ${10} --online-monitoring $9 \
    --write-data ${12} --run-event-count ${13} \
    --run-duration ${14} --file-size ${15} \
    --file-event-count ${17} --file-duration ${18} \
    --run-number $2 2>&1 | tee -a ${11}
}

scriptName=`basename $0`
usage () {
    echo "
Usage: ${scriptName} [options] <command>
Where command is one of:
  init, start, pause, resume, stop, status,
  shutdown, start-system, restart, reinit, exit,
  fast-shutdown, fast-restart, fast-reinit, or fast-exit
General options:
  -h, --help: prints this usage message
Configuration options (init commands):
  -m <on|off>: specifies whether to run online monitoring [default=off]
  -D : disables the writing of data to disk
  -s <file size>: specifies the size threshold for closing data files (in MB)
      [default is 8000 MB (~7.8 GB); zero means that there is no file size limit]
  --file-events <count>: specifies the desired number of events in each file
      [default=0, which means no event count limit for files]
  --file-duration <duration>: specifies the desired duration of each file (minutes)
      [default=0, which means no duration limit for files]
  -c <compression level>: specifies the ADC data compression level
      0 = no compression
      1 = compression, both raw and compressed data kept
      2 = compression, only compressed data kept [default]
  -o <data dir>: specifies the directory for data files [default=/tmp]
Begin-run options (start command):
  -N <run number>: specifies the run number
End-run options (stop command):
  -n, --run-events <count>: specifies the desired number of events in the run
      [default=0, which ends the run immediately, independent of the number of events]
  -d, --run-duration <duration>: specifies the desired length of the run (minutes)
      [default=0, which ends the run immediately, independent of the duration of the run]
Notes:
  The start command expects a run number to be specified (-N).
  The primary commands are the following:
   * init - initializes (configures) the DAQ processes
   * start - starts a run
   * pause - pauses the run
   * resume - resumes the run
   * stop - stops the run
   * status - checks the status of each DAQ process
  Additional commands include:
   * shutdown - stops the run (if one is going), resets the DAQ processes
       to their ground state (if needed), and stops the MPI program (DAQ processes)
   * start-system - starts the MPI program (the DAQ processes)
   * restart - this is the same as a shutdown followed by a start-system
   * reinit - this is the same as a shutdown followed by a start-system and an init
   * exit - this resets the DAQ processes to their ground state, stops the MPI
       program, and exits PMT.
  Expert-level commands:
   * fast-shutdown - stops the MPI program (all DAQ processes) no matter what
       state they are in. This could have bad consequences if a run is going!
   * fast-restart - this is the same as a fast-shutdown followed by a start-system
   * fast-reinit - this is the same as a fast-shutdown followed by a start-system
       and an init
   * fast-exit - this stops the MPI program, and exits PMT.
Examples: ${scriptName} -p 32768 init
          ${scriptName} -N 101 start
" >&2
}

# parse the command-line options
originalCommand="$0 $*"
pulserRate=""
laserRate=""
randomTrigger=""
gateWidth="400"
lowThreshold=0
compressionLevel=2
onmonEnable=off
diskWriting=1
dataDir="/tmp"
runNumber=""
runEventCount=0
runDuration=0
fileSize=8000
fileEventCount=0
fileDuration=0
preTriggerWindow=""
OPTIND=1
while getopts "hc:N:o:m:Dn:d:s:-:" opt; do
    if [ "$opt" = "-" ]; then
        opt=$OPTARG
    fi
    case $opt in
        h | help)
            usage
            exit 1
            ;;
        c)
            compressionLevel=${OPTARG}
            ;;
        N)
            runNumber=${OPTARG}
            ;;
        m)
            onmonEnable=${OPTARG}
            ;;
        o)
            dataDir=${OPTARG}
            ;;
        D)
            diskWriting=0
            ;;
        n)
            runEventCount=${OPTARG}
            ;;
        d)
            runDuration=${OPTARG}
            ;;
        run-events)
            runEventCount=${!OPTIND}
            let OPTIND=$OPTIND+1
            ;;
        run-duration)
            runDuration=${!OPTIND}
            let OPTIND=$OPTIND+1
            ;;
        file-events)
            fileEventCount=${!OPTIND}
            let OPTIND=$OPTIND+1
            ;;
        file-duration)
            fileDuration=${!OPTIND}
            let OPTIND=$OPTIND+1
            ;;
        s)
            fileSize=${OPTARG}
            ;;
        *)
            usage
            exit 1
            ;;
    esac
done
shift $(($OPTIND - 1))

# fetch the command to run
if [ $# -lt 1 ]; then
    usage
    exit
fi
command=$1
shift

# verify that the command is one that we expect
if [[ "$command" != "start-system" ]] && \
   [[ "$command" != "init" ]] && \
   [[ "$command" != "start" ]] && \
   [[ "$command" != "pause" ]] && \
   [[ "$command" != "resume" ]] && \
   [[ "$command" != "stop" ]] && \
   [[ "$command" != "status" ]] && \
   [[ "$command" != "shutdown" ]] && \
   [[ "$command" != "fast-shutdown" ]] && \
   [[ "$command" != "restart" ]] && \
   [[ "$command" != "fast-restart" ]] && \
   [[ "$command" != "reinit" ]] && \
   [[ "$command" != "fast-reinit" ]] && \
   [[ "$command" != "exit" ]] && \
   [[ "$command" != "fast-exit" ]]; then
    echo "Invalid command."
    usage
    exit
fi

# verify that the expected arguments were provided
if [[ "$command" == "start" ]] && [[ "$runNumber" == "" ]]; then
    echo ""
    echo "*** A run number needs to be specified."
    usage
    exit
fi

# fill in values for options that weren't specified
if [[ "$pulserRate" == "" ]]; then
    pulserRate=0
fi
if [[ "$laserRate" == "" ]]; then
    laserRate=0
fi
let laserRate=$laserRate+0
if [[ "$randomTrigger" == "" ]]; then
    randomTrigger=0
fi
if [[ "$runNumber" == "" ]]; then
    runNumber=101
fi
if [[ "$preTriggerWindow" == "" ]]; then
    if [[ $laserRate -ne 0 ]]; then
        preTriggerWindow=1
    else
        preTriggerWindow=5
    fi
fi

# translate the onmon enable flag
if [[ "$onmonEnable" == "on" ]]; then
    onmonEnable=1
else
    onmonEnable=0
fi

# verify that we don't have both a number of events and a time limit
# for stopping a run
if [[ "$command" == "stop" ]]; then
    if [[ "$runEventCount" != "0" ]] && [[ "$runDuration" != "0" ]]; then
        echo ""
        echo "*** Only one of event count or run duration may be specified."
        usage
        exit
    fi
fi

# build the logfile name
TIMESTAMP=`date '+%Y%m%d%H%M%S'`
logFile="/daqlogs/masterControl/dsMC-${TIMESTAMP}-${command}.log"
echo "${originalCommand}" > $logFile
echo ">>> ${originalCommand}"

# invoke the requested command
if [[ "$command" == "shutdown" ]]; then
    # first send a stop command to end the run (in case it is needed)
    launch "stop" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
    # next send a shutdown command to move the processes to their ground state
    launch "shutdown" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
    # stop the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    # clean up any stale shared memory segment
    rsh ${AGGREGATOR_NODE} "ipcs | grep 0x4f4d4f4e | awk '{print \$2}' | xargs ipcrm -m 2>/dev/null"
elif [[ "$command" == "start-system" ]]; then
    rsh ${AGGREGATOR_NODE} "ipcs | grep 0x4f4d4f4e | awk '{print \$2}' | xargs ipcrm -m 2>/dev/null"
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
elif [[ "$command" == "restart" ]]; then
    # first send a stop command to end the run (in case it is needed)
    launch "stop" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
    # next send a shutdown command to move the processes to their ground state
    launch "shutdown" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
    # stop the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    # clean up any stale shared memory segment
    rsh ${AGGREGATOR_NODE} "ipcs | grep 0x4f4d4f4e | awk '{print \$2}' | xargs ipcrm -m 2>/dev/null"
    # start the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
elif [[ "$command" == "reinit" ]]; then
    # first send a stop command to end the run (in case it is needed)
    launch "stop" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
    # next send a shutdown command to move the processes to their ground state
    launch "shutdown" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
    # stop the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    # clean up any stale shared memory segment
    rsh ${AGGREGATOR_NODE} "ipcs | grep 0x4f4d4f4e | awk '{print \$2}' | xargs ipcrm -m 2>/dev/null"
    # start the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
    # send the init command to re-initialize the system
    sleep 5
    launch "init" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
elif [[ "$command" == "exit" ]]; then
    launch "shutdown" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    xmlrpc localhost:${pmtPort}/RPC2 pmt.exit
    rsh ${AGGREGATOR_NODE} "ipcs | grep 0x4f4d4f4e | awk '{print \$2}' | xargs ipcrm -m 2>/dev/null"

elif [[ "$command" == "fast-shutdown" ]]; then
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    rsh ${AGGREGATOR_NODE} "ipcs | grep 0x4f4d4f4e | awk '{print \$2}' | xargs ipcrm -m 2>/dev/null"
elif [[ "$command" == "fast-restart" ]]; then
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    rsh ${AGGREGATOR_NODE} "ipcs | grep 0x4f4d4f4e | awk '{print \$2}' | xargs ipcrm -m 2>/dev/null"
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
elif [[ "$command" == "fast-reinit" ]]; then
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    rsh ${AGGREGATOR_NODE} "ipcs | grep 0x4f4d4f4e | awk '{print \$2}' | xargs ipcrm -m 2>/dev/null"
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
    sleep 5
    launch "init" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
elif [[ "$command" == "fast-exit" ]]; then
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    xmlrpc localhost:${pmtPort}/RPC2 pmt.exit
    rsh ${AGGREGATOR_NODE} "ipcs | grep 0x4f4d4f4e | awk '{print \$2}' | xargs ipcrm -m 2>/dev/null"

else
    launch $command $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $runEventCount $runDuration $fileSize \
        $preTriggerWindow $fileEventCount $fileDuration
fi
